#ifndef _LINUX_CRYPTO_H
#define _LINUX_CRYPTO_H

/*
 * include/linux/crypto.h
 *
 * Written by Alexander Kjeldaas <astor@fast.no> 1998-10-13
 *
 * Copyright 1998 by Alexander Kjeldaas. This code is licensed under
 * an X11-like license.  See LICENSE.crypto for details.
 *
 */

#define CIPHER_NONE     0
#define CIPHER_XOR      1
#define CIPHER_DES      2
#define CIPHER_FISH2    3
#define CIPHER_BLOWFISH 4
#define CIPHER_CAST128  5
#define CIPHER_IDEA     6
#define CIPHER_SERPENT  7
#define CIPHER_MARS     8

#define CIPHER_SKIPJACK 10
#define CIPHER_RC6      11
#define CIPHER_3DES     12
#define CIPHER_E2       13
#define CIPHER_CAST256  14
#define CIPHER_DFC      15
#define CIPHER_RIJNDAEL 16
#define CIPHER_RC5      17
#define MAX_CIPHER	20

#define CIPHER_CBC    0x00010000
#define CIPHER_cbc_DES      (CIPHER_DES      | CIPHER_CBC)
#define CIPHER_cbc_FISH2    (CIPHER_FISH2    | CIPHER_CBC)
#define CIPHER_cbc_BLOWFISH (CIPHER_BLOWFISH | CIPHER_CBC)
#define CIPHER_cbc_CAST128  (CIPHER_CAST128  | CIPHER_CBC)
#define CIPHER_cbc_IDEA     (CIPHER_IDEA     | CIPHER_CBC)
#define CIPHER_cbc_SERPENT  (CIPHER_SERPENT  | CIPHER_CBC)
#define CIPHER_cbc_MARS     (CIPHER_MARS     | CIPHER_CBC)
#define CIPHER_cbc_SKIPJACK (CIPHER_SKIPJACK | CIPHER_CBC)
#define CIPHER_cbc_RC5      (CIPHER_RC5      | CIPHER_CBC)
#define CIPHER_cbc_RC6      (CIPHER_RC6      | CIPHER_CBC)
#define CIPHER_cbc_3DES     (CIPHER_3DES     | CIPHER_CBC)
#define CIPHER_cbc_E2       (CIPHER_E2       | CIPHER_CBC)
#define CIPHER_cbc_CAST256  (CIPHER_CAST256  | CIPHER_CBC)
#define CIPHER_cbc_DFC      (CIPHER_DFC      | CIPHER_CBC)
#define CIPHER_cbc_RIJNDAEL (CIPHER_RIJNDAEL | CIPHER_CBC)


#ifdef __KERNEL__

#include <linux/lists.h>
#include <linux/kernel.h>
#include <asm/page.h>
#include <asm/semaphore.h>
#include <asm/spinlock.h>

typedef u32 u4byte;
typedef u8  u1byte;

struct cipher_context;
	
struct cipher_implementation {
	DLNODE(struct cipher_implementation) link;
	int cipher_id;
	char *cipher_name;
	int blocksize;         /* in bytes */
	int ivsize;            /* in bytes */
	int key_schedule_size; /* in bytes */
	int (*encrypt)(struct cipher_context *cx, 
		       u32 *in, u32 *out, int size);
	int (*decrypt)(struct cipher_context *cx,
		       u32 *in, u32 *out, int size);
	int (*set_key)(struct cipher_context *cx, 
		       char *key, int key_len);

	/* lock and unlock manage the module use counts */ 
	void (*lock)(void);
	void (*unlock)(void);

	/* keep track of the allocated proc_dir_entry */
	struct proc_dir_entry *de;
};

#define MAX_KEY_SIZE 8
#define MAX_IV_SIZE  MAX_KEY_SIZE

struct cipher_context {
	struct cipher_implementation *ci;
	u32 *keyinfo;
	int key_length;              /* in bytes */
	u32 key[MAX_KEY_SIZE];
	u32 iv[MAX_IV_SIZE];
};

typedef void (cipher_iterator)(struct cipher_implementation *);

extern int ciphers_init(void);
extern struct cipher_implementation *find_cipher_by_id(int id);
extern struct cipher_implementation *find_cipher_by_name(char *name);
extern void for_each_cipher(cipher_iterator *fn);
extern int register_cipher(struct cipher_implementation *ci);
extern int unregister_cipher(struct cipher_implementation *ci);
extern int lock_cipher(struct cipher_implementation *ci);
extern int unlock_cipher(struct cipher_implementation *ci);


extern int init_cast256(void);
#define CAST256_KEY_SCHEDULE_SIZE (96*sizeof(u32))
extern  int cast256_set_key(struct cipher_context *cx, char *key, int key_len);
extern  int cast256_encrypt(struct cipher_context *cx, 
			    u32 *in, u32 *out, int size);
extern  int cast256_decrypt(struct cipher_context *cx, 
			    u32 *in, u32 *out, int size);

extern int init_crypton(void);
#define CRYPTON_KEY_SCHEDULE_SIZE (104*sizeof(u32))
extern  int crypton_set_key(struct cipher_context *cx, char *key, int key_len);
extern  int crypton_encrypt(struct cipher_context *cx, 
			    u32 *in, u32 *out, int size);
extern  int crypton_decrypt(struct cipher_context *cx,
			    u32 *in, u32 *out, int size);

extern int init_serpent(void);
#define SERPENT_KEY_SCHEDULE_SIZE (140*sizeof(u32))
extern  int serpent_set_key(struct cipher_context *cx, char *key, int key_len);
extern  int serpent_encrypt(struct cipher_context *cx,
			    u32 *in, u32 *out, int size);
extern  int serpent_decrypt(struct cipher_context *cx, 
			    u32 *in, u32 *out, int size);

extern int init_mars(void);
#define MARS_KEY_SCHEDULE_SIZE (40*sizeof(u32))
extern int mars_set_key(struct cipher_context *cx, char *key, int key_len);
extern int mars_encrypt(struct cipher_context *cx, 
			u32 *in, u32 *out, int size);
extern int mars_decrypt(struct cipher_context *cx, 
			u32 *in, u32 *out, int size);

extern int init_rc5(void);
#define RC5_KEY_SCHEDULE_SIZE (34*sizeof(u32))
extern int rc5_set_key(struct cipher_context *cx, char *key, int key_len);
extern int rc5_encrypt(struct cipher_context *cx, 
		       u32 *in, u32 *out, int size);
extern int rc5_decrypt(struct cipher_context *cx, 
		       u32 *in, u32 *out, int size);

extern int init_rc6(void);
#define RC6_KEY_SCHEDULE_SIZE (44*sizeof(u32))
extern int rc6_set_key(struct cipher_context *cx, char *key, int key_len);
extern int rc6_encrypt(struct cipher_context *cx, 
		       u32 *in, u32 *out, int size);
extern int rc6_decrypt(struct cipher_context *cx, 
		       u32 *in, u32 *out, int size);

extern int init_dfc(void);
#define DFC_KEY_SCHEDULE_SIZE (32*sizeof(u32))
extern int dfc_set_key(struct cipher_context *cx, char *key, int key_len);
extern int dfc_encrypt(struct cipher_context *cx, 
		       u32 *in, u32 *out, int size);
extern int dfc_decrypt(struct cipher_context *cx, 
		       u32 *in, u32 *out, int size);

extern int init_rijndael(void);
#define RIJNDAEL_KEY_SCHEDULE_SIZE ((60+60)*sizeof(u32))
extern int rijndael_set_key(struct cipher_context *cx, char *key, int key_len);
extern int rijndael_encrypt(struct cipher_context *cx, 
		       u32 *in, u32 *out, int size);
extern int rijndael_decrypt(struct cipher_context *cx, 
		       u32 *in, u32 *out, int size);

extern int init_blowfish(void);
#define BLOWFISH_KEY_SCHEDULE_SIZE ((18+1024)*sizeof(u32))
extern int blowfish_set_key(struct cipher_context *cx,
                            unsigned char *key, int key_len);
extern int blowfish_encrypt(struct cipher_context *cx,
							u32 *in, u32 *out, int size);
extern int blowfish_decrypt(struct cipher_context *cx,
                            u32 *in, u32 *out, int size);
extern int init_idea(void);
#define IDEA_KEY_SCHEDULE_SIZE (104*2)
extern int idea_set_key(struct cipher_context *cx,
						char *key, int key_len);
extern int idea_encrypt(struct cipher_context *cx,
						u32 *in, u32 *out, int size);
extern int idea_decrypt(struct cipher_context *cx,
						u32 *in, u32 *out, int size);

extern int init_des(void);
#define DES_KEY_SCHEDULE_SIZE (32*sizeof(u32))
extern int des_set_key(struct cipher_context *cx,
					   unsigned char *key, int key_len);
extern int des_encrypt(struct cipher_context *cx,
					   u32 *in, u32 *out, int size);
extern int des_decrypt(struct cipher_context *cx,
					   u32 *in, u32 *out, int size);

#define DEFINE_GEN_CIPHER(uppername, name, sname, blocksize, ivsize, mode) \
static struct cipher_implementation mode##name =			   \
{									   \
	DNODE_NULL,							   \
	CIPHER_##mode##uppername,					   \
	sname,								   \
	blocksize,							   \
	ivsize,								   \
	uppername##_KEY_SCHEDULE_SIZE,					   \
	name##_##mode##encrypt,						   \
	name##_##mode##decrypt,						   \
	name##_set_key,							   \
	name##_lock,							   \
	name##_unlock							   \
}

#define DEFINE_CIPHER(uppername, name, sname, blocksize, ivsize)	\
        DEFINE_GEN_CIPHER(uppername, name, sname, blocksize, ivsize, )

#define DEFINE_CBC_CIPHER(uppername, name, sname, ivsize)		\
        DEFINE_GEN_CIPHER(uppername, name, sname, 0, ivsize, cbc_)

#define byte(x, nr) ((x) >> (nr*8))

#endif /* __KERNEL__ */

#endif
